/*
 * Copyright (C) 1998, 1999, 2000, 2001
 * Lehrstuhl fuer Technische Informatik, RWTH-Aachen, Germany
 *
 * This file is part of the LTI-Computer Vision Library (LTI-Lib)
 *
 * The LTI-Lib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License (LGPL)
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * The LTI-Lib is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the LTI-Lib; see the file LICENSE.  If
 * not, write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 */


/*----------------------------------------------------------------
 * project ....: LTI Digital Image/Signal Processing Library
 * file .......: ltiUnsupervisedClassifier.cpp
 * authors ....: Peter Doerfler
 * organization: LTI, RWTH Aachen
 * creation ...: 29.08.2001
 * revisions ..: $Id: ltiUnsupervisedClassifier.cpp,v 1.9 2003/04/16 08:25:05 alvarado Exp $
 */

#include "ltiClassifier.h"
#include "ltiUnsupervisedClassifier.h"
#include <ctime>

namespace lti {

  unsupervisedClassifier::unsupervisedClassifier()
    : classifier() {}

  unsupervisedClassifier::unsupervisedClassifier(const unsupervisedClassifier& other)
    : classifier(other)  {}

  // returns the name of this type
  const char* unsupervisedClassifier::getTypeName() const {
    return "unsupervisedClassifier";
  }

  // copy member
  unsupervisedClassifier&
    unsupervisedClassifier::copy(const unsupervisedClassifier& other) {

    classifier::copy(other);
    return (*this);

  }

  // return parameters
  const unsupervisedClassifier::parameters&
  unsupervisedClassifier::getParameters() const {
    const parameters* par =
      dynamic_cast<const parameters*>(&classifier::getParameters());
    if(isNull(par)) {
      throw invalidParametersException(getTypeName());
    }
    return *par;
  }

  bool unsupervisedClassifier::train(const dmatrix& data,
                                     ivector& ids) {

    train(data);

    int nbData=data.rows();

    // resize and clear ids
    ids.resize(nbData, 0, false);

    classifier::outputVector out;

    for (int i=0; i<nbData; i++) {
      if (classify(data.getRow(i),out)) {
        ids.at(i)=out.maxId();
      } else {
        ids.at(i)=classifier::noObject;
      }
    }

    return true;
  }

  bool unsupervisedClassifier::selectRandomPoints(const dmatrix& data,
                                                  int numberOfPoints,
                                                  dmatrix& randomPoints) {

    randomPoints.resize(numberOfPoints, data.columns());
    int i;

    int nbDataPoints=data.rows();
    ivector index(nbDataPoints);
    for (i=0; i<nbDataPoints; i++) {
      index.at(i)=i;
    }

    srand(time(0));
    int rValue;
    int pointsLeft=nbDataPoints;
    for (i=0; i<numberOfPoints; i++) {

      int r=rand();
      rValue=r%(pointsLeft);

      randomPoints.setRow(i, data.getRow(index.at(rValue)));
      index.at(rValue)=index.at(pointsLeft-1);
      pointsLeft--;
      if (pointsLeft==0) {
        pointsLeft=nbDataPoints;
      }
    }

    return true;
  }

}


