/*
 * Copyright (C) 1998, 1999, 2000, 2001
 * Lehrstuhl fuer Technische Informatik, RWTH-Aachen, Germany
 *
 * This file is part of the LTI-Computer Vision Library (LTI-Lib)
 *
 * The LTI-Lib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License (LGPL)
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * The LTI-Lib is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the LTI-Lib; see the file LICENSE.  If
 * not, write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 */


/*----------------------------------------------------------------
 * project ....: LTI Digital Image/Signal Processing Library
 * file .......: ltiGradientKernels_template.h
 * authors ....: Pablo Alvarado
 * organization: LTI, RWTH Aachen
 * creation ...: 11.07.00
 * revisions ..: $Id: ltiGradientKernels_template.h,v 1.8 2003/04/14 08:09:08 sltilib Exp $
 */

#include "ltiMath.h"

namespace lti {
  // constructor
  template<class T>
    gradientKernelX<T>::gradientKernelX(const int& size)
    : sepKernel<T>() {
    generate(size);
  }

  // generate filter
  template<class T>
    void  gradientKernelX<T>::generate(const int& size) {

    setNumberOfPairs(1);

    if ((size>=3) && (size<=5)) {
      int i,l,h;
      l = -size/2;
      h = l+size-1;

      getRowFilter(0).resize(l,h,T(),false,false);
      getColFilter(0).resize(l,h,T(),false,false);

      const T rowNorm = getRowFilter(0).getNorm();
      const T colNorm = getColFilter(0).getNorm();

      static const double k3row[3] = {1.000000,0.000000,-1.000000};
      static const double k3col[3] = { 0.112737,0.274526,0.112737};

      static const double k4row[4] =
        {+0.870541 , +1.000000,  -1.000000, -0.870541};
      static const double k4col[4] =
        { 0.0254738,  0.112996,  0.112996, 0.0254738};

      static const double k5row[5] =
        {+0.379022,+1.000000, 0.000000, -1.000000, -0.379022};
      static const double k5col[5] =
        { 0.0101693, 0.0708223, 0.122602, 0.0708223, 0.0101693};

      switch(size) {
        case 3:
          for (i=0;i<size;i++) {
            getRowFilter(0).at(i-1) = static_cast<T>(k3row[i])*rowNorm;
            getColFilter(0).at(i-1) = static_cast<T>(k3col[i])*colNorm;
          }
          break;
        case 4:
          for (i=0;i<size;i++) {
            getRowFilter(0).at(i-2) = static_cast<T>(k4row[i])*rowNorm;
            getColFilter(0).at(i-2) = static_cast<T>(k4col[i])*colNorm;
          }
          break;
        case 5:
          for (i=0;i<size;i++) {
            getRowFilter(0).at(i-2) = static_cast<T>(k5row[i])*rowNorm;
            getColFilter(0).at(i-2) = static_cast<T>(k5col[i])*colNorm;
          }
          break;
      }
    }
  }

  // constructor
  template<class T>
    gradientKernelY<T>::gradientKernelY(const int& size)
    : sepKernel<T>() {
    generate(size);
  }

  // generate filter
  template<class T>
    void  gradientKernelY<T>::generate(const int& size) {

    setNumberOfPairs(1);

    if ((size>=3) && (size<=5)) {
      int i,l,h;
      l = -size/2;
      h = l+size-1;

      getRowFilter(0).resize(l,h,T(),false,false);
      getColFilter(0).resize(l,h,T(),false,false);

      const T rowNorm = getRowFilter(0).getNorm();
      const T colNorm = getColFilter(0).getNorm();

      static const double k3col[3] = {+1.000000,0.000000,-1.000000};
      static const double k3row[3] = { 0.112737,0.274526,0.112737};

      static const double k4col[4] =
        {+0.870541 , +1.000000,  -1.000000, -0.870541};
      static const double k4row[4] =
        { 0.0254738,  0.112996,  0.112996, 0.0254738};

      static const double k5col[5] =
        {+0.379022,+1.000000, 0.000000, -1.000000, -0.379022};
      static const double k5row[5] =
        { 0.0101693, 0.0708223, 0.122602, 0.0708223, 0.0101693};

      switch(size) {
        case 3:
          for (i=0;i<size;i++) {
            getRowFilter(0).at(i-1) = static_cast<T>(k3row[i])*rowNorm;
            getColFilter(0).at(i-1) = static_cast<T>(k3col[i])*colNorm;
          }
          break;
        case 4:
          for (i=0;i<size;i++) {
            getRowFilter(0).at(i-2) = static_cast<T>(k4row[i])*rowNorm;
            getColFilter(0).at(i-2) = static_cast<T>(k4col[i])*colNorm;
          }
          break;
        case 5:
          for (i=0;i<size;i++) {
            getRowFilter(0).at(i-2) = static_cast<T>(k5row[i])*rowNorm;
            getColFilter(0).at(i-2) = static_cast<T>(k5col[i])*colNorm;
          }
          break;
      }
    }
  }

}

