/*
 * Copyright (C) 1998, 1999, 2000, 2001
 * Lehrstuhl fuer Technische Informatik, RWTH-Aachen, Germany
 *
 * This file is part of the LTI-Computer Vision Library (LTI-Lib)
 *
 * The LTI-Lib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License (LGPL)
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * The LTI-Lib is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the LTI-Lib; see the file LICENSE.  If
 * not, write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 */


/*----------------------------------------------------------------
 * project ....: LTI Digital Image/Signal Processing Library
 * file .......: ltiConstants.h
 * authors ....: Pablo Alvarado
 * organization: LTI, RWTH Aachen
 * creation ...: 16.05.01
 * revisions ..: $Id: ltiConstants.h,v 1.2 2003/06/02 06:30:06 alvarado Exp $
 */

#ifndef _LTI_CONSTANTS_H_
#define _LTI_CONSTANTS_H_

#include "ltiObject.h"
#include "ltiTypes.h"

/**
 * \file ltiConstants.h
 * Definition of a template class containing usual mathematical and
 * physical constants.
 */

namespace lti {

  /**
   * This class is a container of some mathematical and physical constants.
   *
   * Example:
   * \code
   * constants<float>::Pi
   * \endcode
   */
  template<class T>
  class constants : public object {
  public:
    /**
     * Pi
     */
    static const T Pi;

    /**
     * Twice Pi (2*Pi)
     */
    static const T TwicePi;

    /**
     * Half Pi (Pi/2)
     */
    static const T HalfPi;

    /**
     * e (basis of natural logarithm)
     */
    static const T e;
 
  protected:
    /**
     * Protected constructor avoids the creation of
     * an instance of this class.
     */
    constants();
  };
}


#endif

