/*
 * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
 * Lehrstuhl fuer Technische Informatik, RWTH-Aachen, Germany
 *
 * This file is part of the LTI-Computer Vision Library (LTI-Lib)
 *
 * The LTI-Lib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License (LGPL)
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * The LTI-Lib is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the LTI-Lib; see the file LICENSE.  If
 * not, write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 */


/*--------------------------------------------------------------------
 * project ....: LTI-Lib: Image Processing and Computer Vision Library
 * file .......: //@filename
 * authors ....: //@author
 * organization: LTI, RWTH Aachen
 * creation ...: //@date
 * //@rev
 */

#ifndef //@_LTI_FILENAME_
#define //@_LTI_FILENAME_

//@TODO: include only those files which are needed in this header!!
//@includes

namespace lti {
  //@TODO: comment your class
  /**
   *  /*@author*/ should document this!
   */
  class /*@classname*/ : public /*@parentclass*/ {
  public:
    /**
     * The parameters for the class /*@classname*/
     */
    class parameters : public /*@parentclass*/::parameters {
    public:
      /**
       * Default constructor
       */
      parameters();

      /**
       * Copy constructor
       * @param other the parameters object to be copied
       */
      parameters(const parameters& other);

      /**
       * Destructor
       */
      virtual ~parameters();

      /**
       * Returns name of this type
       */
      const char* getTypeName() const;

      /**
       * Copy the contents of a parameters object
       * @param other the parameters object to be copied
       * @return a reference to this parameters object
       */
      parameters& copy(const parameters& other);

      /**
       * Copy the contents of a parameters object
       * @param other the parameters object to be copied
       * @return a reference to this parameters object
       */
      parameters& operator=(const parameters& other);


      /**
       * Returns a pointer to a clone of the parameters
       */
      virtual classifier::parameters* clone() const;

      /**
       * Write the parameters in the given ioHandler
       * @param handler the ioHandler to be used
       * @param complete if true (the default) the enclosing begin/end will
       *        be also written, otherwise only the data block will be written.
       * @return true if write was successful
       */
      virtual bool write(ioHandler& handler,const bool& complete=true) const;

      /**
       * Read the parameters from the given ioHandler
       * @param handler the ioHandler to be used
       * @param complete if true (the default) the enclosing begin/end will
       *        be also written, otherwise only the data block will be written.
       * @return true if write was successful
       */
      virtual bool read(ioHandler& handler,const bool& complete=true);

#     ifdef _LTI_MSC_6
      /**
       * This function is required by MSVC only, as a workaround for a
       * very awful bug, which exists since MSVC V.4.0, and still by
       * V.6.0 with all bugfixes (so called "service packs") remains
       * there...  This method is also public due to another bug, so please
       * NEVER EVER call this method directly: use read() instead
       */
      bool readMS(ioHandler& handler,const bool& complete=true);

      /**
       * This function is required by MSVC only, as a workaround for a
       * very awful bug, which exists since MSVC V.4.0, and still by
       * V.6.0 with all bugfixes (so called "service packs") remains
       * there...  This method is also public due to another bug, so please
       * NEVER EVER call this method directly: use write() instead
       */
      bool writeMS(ioHandler& handler,const bool& complete=true) const;
#     endif

      // ------------------------------------------------
      // the parameters
      // ------------------------------------------------

      //@TODO: comment the parameters of your classifier
      // If you add more parameters manually, do not forget to do following:
      // 1. indicate in the default constructor the default values
      // 2. make sure that the copy member also copy your new parameters
      // 3. make sure that the read and write members also read and
      //    write your parameters
      /*@parameters*/

    };

    /**
     * Default constructor.
     */
    /*@classname*/();

    /**
     * Copy constructor.
     * @param other the object to be copied
     */
    /*@classname*/(const /*@classname*/& other);

    /**
     * Destructor
     */
    virtual ~/*@classname*/();

    /**
     * Returns the name of this type ("/*@classname*/")
     */
    virtual const char* getTypeName() const;

    /**
     * Copy data of "other" classifier.
     * @param other the classifier to be copied
     * @return a reference to this classifier object
     */
    /*@classname*/& copy(const /*@classname*/& other);

    /**
     * Alias for copy member
     * @param other the classifier to be copied
     * @return a reference to this classifier object
     */
    /*@classname*/& operator=(const /*@classname*/& other);

    /**
     * Returns a pointer to a clone of this classifier.
     */
    virtual classifier* clone() const;

    /**
     * Returns used parameters
     */
    const parameters& getParameters() const;

    /**
     * Supervised sequence training.
     * The sequences in the <code>input</code> %vector are used for training
     * the %sequence %classifier. Each %sequence is associated with an id
     * given in the %vector <code>ids</code>.
     * @param input %vector of training %sequences
     * @param ids the classes of the training sequences
     * @return true if successful, false otherwise.  (if false you can check
     *              the error message with getStatusString())
     */
    virtual bool train(const std::vector< sequence<dvector> > input,
                       const ivector& ids);

    //@TODO Check whether you really need a new classify method.
    // In some cases the superclasses method will suffice. Then just
    // delete the declaration and its implementation stump.

    /**
     * Classification.
     * Classifies the %sequence of features and returns the outputVector with
     * the classification result. Since %sequence classifiers usually keep
     * an inner state, this method is not const.
     * @param observations the %sequence to be classified
     * @param result the result of the classification
     * @return false if an error occurred during classification else true
     */
    virtual bool classify(const sequence<dvector>& observations,
                          outputVector& result);


    //@TODO: comment the attributes of your classifier
    // If you add more attributes manually, do not forget to do following:
    // 1. indicate in the default constructor the default values
    // 2. make sure that the copy member also copy your new attributes, or
    //    to ensure there, that these attributes are properly initialized.

  };
}

#endif

