:: ----------------------------------------------------------------------------
@echo off
@cls
set success=1
set scriptName=%0

:: Setting Visual Studio .NET environment -------------------------------------
:: change the path to vsvars32.bat to match your machine, it is normally-------
:: located in the common7\tools-directory of the Visual Studio home directory--

call "vsvars32.bat"

:: Get commandline options ----------------------------------------------------
set clear=0
set co=0
set nomake=0
set noheaders=0
set help=0
set dir=0

:CLLOOP
  set valid=0
  if "%1"=="" goto :CLEXIT
  if "%1"=="-clear"     set clear=1     & set valid=1
  if "%1"=="-co"        set co=1        & set valid=1
  if "%1"=="-nomake"    set nomake=1    & set valid=1
  if "%1"=="-noheaders" set noheaders=1 & set valid=1
  if "%1"=="-help"      set help=1      & set valid=1
  if "%1"=="-h"         set help=1      & set valid=1
  if NOT "%1"=="-dir" GOTO :NO_DIR
    shift 
    set dir=1
    set dirname=%1
    set valid=1
  :NO_DIR
  
  if "%valid%"=="1" goto :OPTION_OK
    @echo Invalid option: %1
    @echo.
    @echo For help please type: %scriptName% -h
    goto END
  :OPTION_OK  
  
  shift
  goto :CLLOOP
:CLEXIT
:: ----------------------------------------------------------------------------


:: Print Help -----------------------------------------------------------------
if %help%==0 goto :ENDHELP
@echo Builds the ltilib. Creates makefile and collects headerfiles beforehand.
@echo.
@echo Usage: %scriptName% [-clear] [-co] [-nomake] [-noheaders] [-dir "dirname"]
@echo.
@echo   -clear            deletes all existing object files (forces complete rebuild)
@echo   -co               check out lib from repository before compiling
@echo   -nomake           don't create makefile
@echo   -noheaders        don't collect headerfiles
@echo   -dir "dirname"    copy lib to specified directory, if successful
goto :END
:ENDHELP
:: ----------------------------------------------------------------------------


:: Process options ------------------------------------------------------------
@echo.
@echo.
@echo -- Logfile for automatic build of LtiLib --

if %clear%==0 goto :ENDCLEAR
  @echo.
  @echo.
  @echo -- Clear existing configurations ...
  devenv ltilib.sln /clean "MFC Shared Debug"
  devenv ltilib.sln /clean "MFC Shared Release"
  devenv ltilib.sln /clean "MFC Static Debug"
  devenv ltilib.sln /clean "MFC Static Release"
  devenv ltilib.sln /clean "WIN32 Debug"
  devenv ltilib.sln /clean "WIN32 Release"
:ENDCLEAR

if %co%==0 goto :ENDCO
  @cd ..\..
  @echo.
  @echo.
  @echo -- Updating sources from CVS repository ...

:: change the next 3 settings for cvs to match your needs
  set Path=%Path%;"C:\Programme\WinCVS 1.2\"
  set HOME=MyCVSHomeDirectory
  set CVSROOT=:pserver:MyRootToLTILibRepository

  cvs -d :pserver:ltilib@tintas.techinfo.rwth-aachen.de:/home/cvsroot/lti update -d
  if ERRORLEVEL 1 goto :CHECKOUTFAILED
  goto :CHECKOUTOK
  :CHECKOUTFAILED
    @echo Could not check out new sources. Maybe you forgot to adjust the settings on lines 90-92.
  :CHECKOUTOK
  @cd win\buildLib_net
:ENDCO

if %nomake%==1 goto :ENDNOMAKE
  @echo.
  @echo.
  @perl buildLib.net.pl 
:ENDNOMAKE

if %noheaders%==1 goto :ENDNOHEADERS
  @echo.
  @echo.
  @perl collectHeaders.pl 
:ENDNOHEADERS
:: ----------------------------------------------------------------------------


:: Build the lib --------------------------------------------------------------
:: Comment out the configurations you do not need -----------------------------
@echo.
@echo.
REM build debug version of ltilib with MFC shared library support
devenv ltilib.sln /build "MFC Shared Debug"

REM build release version of ltilib with MFC shared library support
devenv ltilib.sln /build "MFC Shared Release"

REM build debug version of ltilib with MFC static library support
REM devenv ltilib.sln /build "MFC Static Debug"

REM build release version of ltilib with MFC shared library support
REM devenv ltilib.sln /build "MFC Static Release"

REM build debug version of ltilib for WIN32
REM devenv ltilib.sln /build "WIN32 Debug"

REM build release version of ltilib for WIN32
REM devenv ltilib.sln /build "WIN32 Release"

:: ----------------------------------------------------------------------------


:: Copy the lib (if desired) --------------------------------------------------
if "%dir%"=="0" GOTO :END
  @echo.
  @echo.
  @echo -- Copy lib to %dirname%

if "%success%"=="1" goto :COPY_LIB
  @echo.
  @echo Build was not successful. Not copying lib!
goto :END

:COPY_LIB
  @echo.
  echo Checking directory %dirname%
  mkdir %dirname% 
  if NOT exist %dirname% goto :DIR_NOT_OK
REM @echo Clearing existing files in %dirname%
REM @if exist %dirname%\lib del /Q /S %dirname%\lib\*.* >NUL 2>NUL
    @echo Copying lib to %dirname% ...
    xcopy ..\..\lib %dirname% /y/s/i/r/k
    @echo Finished copying!
    goto :DIR_OK
  :DIR_NOT_OK
  @echo Couldn't create directory for copying lib
  GOTO :END
:DIR_OK
GOTO :END
:: ----------------------------------------------------------------------------


:: The END --------------------------------------------------------------------
:END
@echo.
@echo --
@echo.
:: ----------------------------------------------------------------------------
